﻿using DevExpress.Data.Filtering;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.Actions;
using DevExpress.ExpressApp.Editors;
using DevExpress.ExpressApp.Model;
using DevExpress.ExpressApp.Web.Editors.ASPx;
using DevExpress.Persistent.Base;
using DevExpress.Web;
using RGPD.Module.BusinessObjects.RGPD;
using RGPD_Manager.Module.BusinessObjects.Non_Persistent;
using System;
using System.Collections.Generic;
using System.Linq;
using Xafari;
using Xafari.Controllers;
using Xafari.Model;

namespace RGPD_Manager.Module.Controllers.Analyse
{
    // For more typical usage scenarios, be sure to check out https://documentation.devexpress.com/eXpressAppFramework/clsDevExpressExpressAppViewControllertopic.aspx.
    public partial class NormesAction : ViewController
    {
        private List<NonPersistentBlocNorme> nonPersistentBlocNormeList;
        ASPxGridListEditor gridListEditor = null;

        public NormesAction()
        {
            InitializeComponent();
            // Target required Views (via the TargetXXX properties) and create their Actions.
        }
        protected override void OnActivated()
        {
            base.OnActivated();
            // Perform various tasks depending on the target View.
        }
        protected override void OnViewControlsCreated()
        {
            base.OnViewControlsCreated();
            // Access and customize the target View control.
            if(View.Id == "Norme_LookupListView_Analyse")
            {
                gridListEditor = ((ListView)View).Editor as ASPxGridListEditor;
                if (gridListEditor != null)
                {
                    gridListEditor.Grid.PreRender += new EventHandler(Grid_PreRender);
                    gridListEditor.Grid.SettingsBehavior.AllowSelectSingleRowOnly = true;
                }
            }
            if(View.Id == "Wizard_Analyser_Step_1")
            {
                WizardDialogController wizardDialogController = Frame.GetController<WizardDialogController>();
                if (wizardDialogController != null)
                {
                    wizardDialogController.AcceptAction.Active["Allow"] = false;
                    wizardDialogController.AcceptAction.Active["Next not active"] = false;
                }           
            }
        }
        protected override void OnDeactivated()
        {
            if (gridListEditor != null && gridListEditor.Grid != null)
            {
                gridListEditor.Grid.PreRender -= Grid_PreRender;
                gridListEditor = null;
            }
            // Unsubscribe from previously subscribed events and release other references and resources.
            base.OnDeactivated();
        }

        private void Grid_PreRender(object sender, EventArgs e)
        {
            foreach (GridViewColumn column in ((ASPxGridView)sender).Columns)
            {
                if (column is GridViewDataActionColumn)
                {
                    if (((GridViewDataActionColumn)column).Action.Id == "Edit")
                    {
                        column.Visible = false;
                    }
                }
            }
        }

        private void FindNorme_CustomizePopupWindowParams(object sender, CustomizePopupWindowParamsEventArgs e)
        {
            IObjectSpace objectSpace = Application.CreateObjectSpace(typeof(Norme));
            string SystemListViewId = "Norme_LookupListView_Analyse";
            CollectionSourceBase collectionSource = Application.CreateCollectionSource(objectSpace, typeof(Norme), SystemListViewId);
            e.View = Application.CreateListView(SystemListViewId, collectionSource, true);
            e.View.Tag = View.CurrentObject;
        }

        private void Analyser_CustomizePopupWindowParams(object sender, CustomizePopupWindowParamsEventArgs e)
        {
            var objectSpace = Application.CreateObjectSpace();

            var controller = XafariModule.CreateWizardDialogController(Application);
            controller.Model = Application.Model.Xafari().Wizards.GetWizard("Analyser");
            controller.CustomNextStep += controller_CustomNextStep;
            e.Context = controller.Model.TemplateContext;

            int indexQuestion = 1;
            nonPersistentBlocNormeList = new List<NonPersistentBlocNorme>();
            Norme currentNorme = (Norme)View.CurrentObject;

            foreach (BlocNorme blocNorme in currentNorme.BlocNormes.OrderBy(i => i.I_Order))
            {
                IModelWizardStep step = (IModelWizardStep)controller.Model.GetNode("Step_" + indexQuestion.ToString());
                if (step == null)
                    step = controller.Model.AddNode<IModelWizardStep>("Step_" + indexQuestion.ToString());

                step.View = (IModelDetailView)Application.FindModelView("Wizard_Analyser_Step_1");
                step.Index = indexQuestion;
                step.Caption = blocNorme.S_Titre;
                step.Description = blocNorme.S_Description;
                step.AllowBack = false;

                NonPersistentBlocNorme tempBlocNorme = new NonPersistentBlocNorme() { Oid = blocNorme.Oid, Ordre = indexQuestion, Question = blocNorme.S_Question };
                nonPersistentBlocNormeList.Add(tempBlocNorme);

                indexQuestion++;
            }

            controller.AcceptAction.Caption = "Terminer";
            controller.AcceptAction.ToolTip = "Terminer";
            //controller.AcceptAction.TargetObjectType = typeof(NonPersistentBlocNorme);
            //controller.AcceptAction.TargetObjectsCriteria = "[Reponse] = True";

            SimpleAction simpleAction = new SimpleAction(controller, "TerminerAnalyse",PredefinedCategory.PopupActions);
            simpleAction.Caption = "Test";
            simpleAction.TargetViewId = "Wizard_Analyser_Step_1";
            simpleAction.TargetObjectType = typeof(NonPersistentBlocNorme);
            simpleAction.TargetObjectsCriteria = "[Reponse] = True";
            simpleAction.ActionMeaning = ActionMeaning.Accept;
            simpleAction.Active["Allow"] = true;
            simpleAction.Active["Next not active"] = false;
            controller.Actions.Add(simpleAction);

            e.DialogController = controller;

            var detailView = Application.CreateDetailView(objectSpace, controller.Model[0].View, true, null);
            detailView.ViewEditMode = ViewEditMode.Edit;
            e.View = detailView;
        }

        private void Analyser_Execute(object sender, PopupWindowShowActionExecuteEventArgs e)
        {
            //Inscrire la norme dans le traitement si valide
            bool validAnalyse = true;
            IObjectSpace objectSpace = View.ObjectSpace;
            foreach (NonPersistentBlocNorme blocNorme in nonPersistentBlocNormeList)
            {
                if(objectSpace.FindObject<BlocNorme>(CriteriaOperator.Parse("Oid=" + blocNorme.Oid)).S_Answer != blocNorme.Reponse)
                {
                    validAnalyse = false;
                    break;
                }   
            }
            
            if(validAnalyse)
            {
                Traitement traitement = objectSpace.GetObject((Traitement)View.Tag);
                Norme norme = objectSpace.GetObject((Norme)View.CurrentObject);
                traitement.FK_Norme = norme;
                objectSpace.CommitChanges();
            }

            //IObjectSpace objectSpace = Application.CreateObjectSpace();
            //objectSpace = Application.CreateObjectSpace(typeof(Traitement));
            //ShowViewParameters viewParams = new ShowViewParameters();
            //CollectionSourceBase collectionSource = Application.CreateCollectionSource(objectSpace, typeof(Traitement), "Traitement_ListView_Statut");
            //ListView view = Application.CreateListView("Traitement_ListView_Statut", collectionSource, true);
            //viewParams.CreatedView = view;
            //viewParams.CreateAllControllers = true;
            //Application.ShowViewStrategy.ShowView(viewParams, new ShowViewSource(null, null));
        }

        void controller_CustomNextStep(object sender, CustomNextStepEventArgs e)
        {
            if (e.WizardObject<NonPersistentBlocNorme>() == null || e.WizardObject<NonPersistentBlocNorme>().Reponse)
            {
                var controller = (WizardDialogController)sender;
                
                var obj = nonPersistentBlocNormeList.First(i => i.Ordre == e.CurrentStep.Index + 1);

                controller.CurrentObject = obj;
                controller.Window.View.CurrentObject = obj;
            }
            else
                e.NextStep = e.CurrentStep;
        }

        private void Analyser_CreateWizardObject(object sender, Xafari.Actions.CreateWizardObjectEventArgs e)
        {
            e.Object = new NonPersistentBlocNorme();
        }

        private void simpleAction1_Execute(object sender, SimpleActionExecuteEventArgs e)
        {
            //var objectSpace = Application.CreateObjectSpace();

            //var controller = XafariModule.CreateWizardDialogController(Application);
            //controller.Model = Application.Model.Xafari().Wizards.GetWizard("Analyser");
            //controller.CustomNextStep += controller_CustomNextStep;

            //int indexQuestion = 1;
            //nonPersistentBlocNormeList = new List<NonPersistentBlocNorme>();
            //Norme currentNorme = (Norme)View.CurrentObject;

            //foreach (BlocNorme blocNorme in currentNorme.BlocNormes.OrderBy(i => i.I_Order))
            //{
            //    IModelWizardStep step = (IModelWizardStep)controller.Model.GetNode("Step_" + indexQuestion.ToString());
            //    if (step == null)
            //        step = controller.Model.AddNode<IModelWizardStep>("Step_" + indexQuestion.ToString());

            //    step.View = (IModelDetailView)Application.FindModelView("Wizard_Analyser_Step_1");
            //    step.Index = indexQuestion;
            //    step.Caption = blocNorme.S_Titre;
            //    step.Description = blocNorme.S_Description;
            //    step.AllowBack = false;

            //    NonPersistentBlocNorme tempBlocNorme = new NonPersistentBlocNorme() { Oid = blocNorme.Oid, Ordre = indexQuestion, Question = blocNorme.S_Question };
            //    nonPersistentBlocNormeList.Add(tempBlocNorme);

            //    indexQuestion++;
            //}

            //controller.AcceptAction.Caption = "Terminer";
            //controller.AcceptAction.ToolTip = "Terminer";

            //e.ShowViewParameters.Controllers.AddRange(new[] { controller });
            //e.ShowViewParameters.Context = controller.Model.TemplateContext;
            //e.ShowViewParameters.CreatedView = Application.CreateDetailView(objectSpace, controller.Model[0].View, true, null);
            //((DetailView)e.ShowViewParameters.CreatedView).ViewEditMode = ViewEditMode.Edit;
            //e.ShowViewParameters.TargetWindow = TargetWindow.NewModalWindow;
            //e.ShowViewParameters.NewWindowTarget = NewWindowTarget.Separate;
        }
    }
}
